/*********************************************************************
 *
 *                 1-Wire Protocol Application developed for DS2411
 *
 *********************************************************************
 * FileName:        demowire.c
 * Dependencies:
 * Processor:       PIC18
 * Complier:        MCC18 v3.13
 * Company:         Microchip Technology, Inc.
 *
 * Software License Agreement
 *
 * Copyright  2004-2007 Microchip Technology Inc.  All rights reserved.
 *
 * Microchip licenses to you the right to use, copy and distribute Software 
 * only when embedded on a Microchip microcontroller or digital signal 
 * controller and used with a Microchip radio frequency transceiver, which 
 * are integrated into your product or third party product (pursuant to the 
 * sublicense terms in the accompanying license agreement).  You may NOT 
 * modify or create derivative works of the Software.  
 *
 *
 * You should refer to the license agreement accompanying this Software for 
 * additional information regarding your rights and obligations.
 *
 * SOFTWARE AND DOCUMENTATION ARE PROVIDED AS IS WITHOUT WARRANTY OF ANY 
 * KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY 
 * OF MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR 
 * PURPOSE. IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE LIABLE OR OBLIGATED 
 * UNDER CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF 
 * WARRANTY, OR OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR 
 * EXPENSES INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, 
 * PUNITIVE OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF 
 * PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY 
 * THIRD PARTIES (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER 
 * SIMILAR COSTS.
 *
 * Author               Date        Comment
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Sasha     			12/20/07    Original
 ********************************************************************/

//** I N C L U D E S **********************************************************/
//#include <p18f4580.h>
#include <p18cxxx.h>
#include <delays.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include "Config.h"
#include "1wire.h"

//******* G E N E R I C   D E F I N I T I O N S ************************************************/
#define BAUD_CONTROL			00
//******* ONE WIRE DEFINITIONS *****************************************************************/
#define SEARCHROM_COMMAND_DS1820 0xF0		 // Command to search for ROM codes
#define READ_COMMAND_DS1820  	0x33         // Command to read the 64-bit serial number from 1-wire slave device.
#define MATCHROM_COMMAND_DS1820 0x55		 // Command to match the SN of the 1-wire slave device
#define IGNOREROM_COMMAND_DS1820 0xCC		 // Command to ignore the SN of the 1-wire slave device
#define ALARMSEARCH_COMMAND_DS1820 0xEC		 // Alarm Search
#define CONVERTT_COMMAND_DS180	0x44		 // Convert Temperature
#define WRITESCRATCHPAD_COMMAND_DS180 0x4E	 // Write to the scratchpad
#define READSCRATCHPAD_COMMAND_DS180 0xBE	 // Read the scratchpad
#define COPYSCRATCHPAD_COMMAND_DS180 0x48	 // Copy the scratchpad
#define RECALLALARM_COMMAND_DS180 0xB8		 // Recall the alarm trigger values
#define READPOWERSUPPLY_COMMAND_DS180 0xB4	 // Read the power supply status

#define OW_ROMCODE_SIZE 8	//Romcode size, 64 bits = 8 bytes
#define OW_SEARCH_FIRST 0xFF
#define OW_LAST_DEVICE 0x00	//0x01 ... 0x40 = keep searching
#define OW_DATA_ERR 0xFE	//Data error
#define OW_PRESENCE_ERR 0xFF	//Presence error
#define MAX_SENSORS_DS1820 6	//Maximum Number Of Sensors

#define DS18S20_ID 0x10	//Device ID
#define DS18B20_ID 0x28	//Device ID
#define DS1822_ID 0x22	//Device ID

#define DS18B20_TCONV_12BIT      750
#define DS18B20_TCONV_11BIT      DS18B20_TCONV_12_BIT/2
#define DS18B20_TCONV_10BIT      DS18B20_TCONV_12_BIT/4
#define DS18B20_TCONV_9BIT       DS18B20_TCONV_12_BIT/8
#define DS18S20_TCONV            DS18B20_TCONV_12_BIT

#define DS1822_TCONV_12BIT      DS18B20_TCONV_12BIT
#define DS1822_TCONV_11BIT      DS18B20_TCONV_12_BIT/2
#define DS1822_TCONV_10BIT      DS18B20_TCONV_12_BIT/4
#define DS1822_TCONV_9BIT       DS18B20_TCONV_12_BIT/8

#define DS18X2X_FRACCONV         625
#define DS18X2X_SP_SIZE  9

#define DS18B20_CONF_REG    4
#define DS18B20_9_BIT       0
#define DS18B20_10_BIT      (1<<5)
#define DS18B20_11_BIT      (1<<6)
#define DS18B20_12_BIT      ((1<<6)|(1<<5))

#define DS18B20_9_BIT_UNDF       ((1<<0)|(1<<1)|(1<<2))
#define DS18B20_10_BIT_UNDF      ((1<<0)|(1<<1))
#define DS18B20_11_BIT_UNDF      ((1<<0))
#define DS18B20_12_BIT_UNDF      0

#define CRC8INIT	0x00
#define CRC8POLY	0x18              //0X18 = X^8+X^5+X^4+X^0

//******* END OF ONE WIRE DEFINITIONS **********************************************************/

//****** V A R I A B L E S ********************************************************/
unsigned char welcome_message[] = "I AM A MASTER DEVICE\r\n";
unsigned char not_present[] = "SLAVE Device(s) is not present\r\n";
unsigned char dev_present [] = "SLAVE Device(s) is present\r\n";
unsigned char ser_num [] = "SERIAL NUMBER:  \r\n";
unsigned char sensor_ids[MAX_SENSORS_DS1820][OW_ROMCODE_SIZE];
//*************************************************************************

//****** P R O T O T Y P E S ******************************************************/
void PutString (unsigned char *mess);
unsigned char Getchar(void);
void Putchar (unsigned char);
unsigned char Detect_Slave_Device(void);
unsigned char Search_Devices(void);
void Show_Temperatures(unsigned int nSensors);
void hex_ascii(unsigned char);

//****** DS1820 Functions **********************************************************/
void DS1820_find_sensor(unsigned char *diff, unsigned char id[]);
void DS18X2X_start_meas(void);
unsigned char DS18X2X_read_meas(unsigned char id[]);
char DS18X2X_get_temp(char *sp);
void DS18X2X_show_id_uart(unsigned char *id, char n);
//****** EBD OF DS1820 Functions ***************************************************/
//****** ONE WIRE Functions ********************************************************/
unsigned char ow_rom_search(unsigned char diff, unsigned char *id);
void ow_command(unsigned char command, unsigned char *id);
unsigned char crc8 (unsigned char* data_in, unsigned int number_of_bytes_to_read);
int abs(int i);
//****** END OF ONE WIRE Functions *************************************************/
//*************************************************************************

/******************************************************************************
/** I N T E R R U P T  V E C T O R S *****************************************/
/******************************************************************************/
#pragma code high_vector=0x08
void interrupt_at_high_vector(void)
{
   _asm goto high_isr _endasm
}
#pragma code

#pragma code low_vector=0x18
void interrupt_at_low_vector(void)
{
   _asm goto low_isr _endasm
}
#pragma code


/******************************************************************************
* Function:        void high_isr(void)
* PreCondition:    None
* Input:		   None
* Output:		   None	
* Side Effects:    None
* Overview:        None
*****************************************************************************/
#pragma interrupt high_isr
void high_isr(void)
{

}

/**********************************************************************
* Function:        void low_isr(void)
* PreCondition:    None
* Input:		   None
* Output:		   None	
* Side Effects:    None
* Overview:        None
***********************************************************************/
#pragma interruptlow low_isr
void low_isr(void)
{
}
#pragma code


/**********************************************************************
* Function:        main function.
* PreCondition:    None
* Input:		   None
* Output:		   None	
* Overview:		   1-wire protocol develop for DS2411
***********************************************************************/
int abs(int i)
{
if (i < 0)
return -i;
else
return i;
} 

void main(void)
{
   unsigned int i = 0;
   unsigned int j = 0;
   unsigned char temp;
   unsigned char temp1;
   unsigned char serial_number[8];
   unsigned char nSensors;
   unsigned char sensormsg[] = " DS1820 Sensor(s) available.\r\n";
   unsigned char busmsg[] = "# in Bus :";

   initEUSART();							// Initilization of USART

   PutString(welcome_message);				// Display Welocme Message on Terminal.

   if(!Detect_Slave_Device())	    		// Is slave present???
   		PutString(not_present);				// Send message to terminal showing 1-wire slave device not present.
   else
   		PutString(dev_present);				// Send message to terminal showing 1-wire slave device present.
	nSensors = Search_Devices();
   	Putchar((char)(48 + nSensors));
   	PutString(sensormsg);
   	for (i=0; i<nSensors; i++) 
    {
		PutString(busmsg);
		Putchar((char) (48 + (i+1)));
		Putchar(0x20);
		Putchar(':');
		Putchar(0x20);
		DS18X2X_show_id_uart(&sensor_ids[i][0], OW_ROMCODE_SIZE);
		Putchar('\r');
    }
	while(1)
	{
		Show_Temperatures(nSensors);
 		Delay10KTCYx(255);
	}   
  
} // END OF MAIN

void Show_Temperatures(unsigned int nSensors)
{
	unsigned char i = 0;
	unsigned char conversionmsg[] = "Convert_T for all Sensors and Read Sensor by Sensor ROM\r\n";
	unsigned char sensormsg[] = "Sensor# ";
	unsigned char crcerrmsg[] = "CRC Error (lost connection ?)";
	PutString(conversionmsg);
    DS18X2X_start_meas();
    Delay10KTCYx(255);	//750(ms) * 1000 / 10000(10K) / (CLOCK / 4) = 20MHz / 4 = 5 = 15!!
    for (i=0; i<nSensors; i++) 
    { 
		PutString(sensormsg);
        Putchar((char)(48 + (i+1)));
        Putchar(0x20);        
		Putchar('=');
        Putchar(0x20);        
        if (DS18X2X_read_meas(&sensor_ids[i][0]) == NULL)
        { 
			;
        }
        else 
			PutString(crcerrmsg);
        Putchar('\r');
     }
}

unsigned char Search_Devices(void)
{
	unsigned int i;
	unsigned char id[OW_ROMCODE_SIZE] = {0};
	unsigned char diff = 0, nSensors = 0;
	unsigned char searchmsg[] = "Searching for sensors...\r\n";
	unsigned char nosensormsg[] = "No Sensor Found.\r\n";
	unsigned char buserrmsg[] = "Bus Error.\r\n";
	PutString(searchmsg);
	for(diff = OW_SEARCH_FIRST; diff != OW_LAST_DEVICE && nSensors < MAX_SENSORS_DS1820;)
	{
		DS1820_find_sensor(&diff, &id[0]);	
		if(diff == OW_PRESENCE_ERR)
		{
			PutString(nosensormsg);
			break;
		}
		if(diff == OW_DATA_ERR)
		{
			PutString(buserrmsg);
			break;
		}
		for(i = 0; i < OW_ROMCODE_SIZE; i++)
		{		
			sensor_ids[nSensors][i] = id[i];
		}
		nSensors++;
	}
	return nSensors;
}

void DS1820_find_sensor(unsigned char *diff, unsigned char id[])
{ 
	for (;;) 
    { 
		*diff = ow_rom_search(*diff, &id[0]);
      	if(*diff==OW_PRESENCE_ERR || *diff==OW_DATA_ERR || *diff == OW_LAST_DEVICE) return;
      	if(id[0] == DS18B20_ID || id[0] == DS18S20_ID || id[0] == DS1822_ID) return;
    }
 }

unsigned char ow_rom_search(unsigned char diff, unsigned char *id)
{
	unsigned char i = 0, j = 0, next_diff = 0;
   	unsigned char b = 0;

   if( OW_reset_pulse() ) 
      return OW_PRESENCE_ERR;       // error, no device found

   OW_write_byte ( SEARCHROM_COMMAND_DS1820 );     // ROM search command
   next_diff = OW_LAST_DEVICE;      // unchanged on last device

   i = OW_ROMCODE_SIZE * 8;         // 8 bytes
   do 
    {	j = 8;                        // 8 bits
		do 
       { b = OW_read_bit();			// read bit
         if( OW_read_bit() ) 
          { // read complement bit
            if( b )                 // 11
               return OW_DATA_ERR;  // data error
          }
         else 
          { if( !b ) 
             { // 00 = 2 devices
               if( diff > i || ((*id & 1) && diff != i) ) 
                { b = 1;               // now 1
                  next_diff = i;       // next pass 0
                }
             }
          }
         OW_write_bit(b);               // write bit
         *id >>= 1;
         if( b ) *id |= 0x80;			// store bit
         i--;
       } 
      while( --j );
      id++;                            // next byte
    } 
   while( i );
   return next_diff;                  // to continue search
}

void DS18X2X_start_meas(void)
{ 
	OW_reset_pulse();
    OW_write_byte(IGNOREROM_COMMAND_DS1820);
	OW_write_byte(CONVERTT_COMMAND_DS180);
	drive_OW_high();
}

unsigned char DS18X2X_read_meas(unsigned char id[])
{
	unsigned int i = 0;
   	char sp[DS18X2X_SP_SIZE];
   	OW_reset_pulse();
   	ow_command(READSCRATCHPAD_COMMAND_DS180, id);
	for ( i=0 ; i< DS18X2X_SP_SIZE; i++ )
	{
		sp[i]=OW_read_byte_s();
	}
  	if(crc8((unsigned char*)&sp[0], DS18X2X_SP_SIZE)) 
    	return 0x01;
   	DS18X2X_get_temp(sp);
   	return 0x00;
}
char DS18X2X_get_temp(char *sp)
{
	unsigned char tempC[] = "000.0\r\n";
	unsigned char tempF[] = "000.0\r\n";
	signed int whole = 0, temp = 0;
	signed int LSB = 0;
	signed int frac = 0;
	int CNeg = 0;
	whole = ((sp[1] << 8) + (sp[0] & 0xff) >> 4);
	LSB = sp[0] & 0xF;
	frac = LSB * 625;
	if (whole & 0x8000) { 
		tempC[0] = '-'; 
		// Negative temp values are stored in 2's complement form 
		whole = ~whole + 1; 
		CNeg = 1;
	}
	else tempC[0] = ' ';
	if ((whole/100) && CNeg == 0)
	{	
		tempC[0] = whole/100 + 48;
		temp = (int)(whole / 100) * 100;
		whole -=temp;
	}
	tempC[1] = (whole/10) + 48;
	temp = (int)(whole / 10) * 10;
	whole -= temp;
	tempC[2] = whole + 48;
	tempC[4] = (frac / 1000) + 48;
	PutString(tempC);
}
/*char DS18X2X_get_temp(char *sp)
{
	// String array to store temperature value to display
	unsigned char tempC[] = "000.0\r\n";
	unsigned char tempF[] = "000.0\r\n";

	// Variables to store temperature register values
	unsigned int temp_whole, temp_fraction, temp_value;
	signed int tempinF, tempinC;
	unsigned short C_Neg=0, F_Neg=0, TempH, TempL;
	temp_value = (sp[1] << 8)+ sp[0] ;
	// check if temperature is negative 
	if (temp_value & 0x8000) { 
		C_Neg = 1; 
		tempC[0] = '-'; 
		// Negative temp values are stored in 2's complement form 
		temp_value = ~temp_value + 1; 
	}
	else C_Neg = 0;
	// Get temp_whole by dividing by 2 
	temp_whole = temp_value >> 1 ;
	if (temp_value & 0x0001){ // LSB is 0.5C
		temp_fraction = 5;
	}
	else temp_fraction = 0;
	tempinC = temp_whole*10+temp_fraction;
	if(C_Neg) {
		tempinF = 320-9*tempinC/5;
		if (tempinF < 0) {
			F_Neg = 1;
			tempF[0] = '-';
			tempinF = abs(tempinF);
		}
		else F_Neg = 0;
	}
	else tempinF = 9*tempinC/5 + 320;
	//--- Format and display result on Lcd

	// convert Temp to characters
	if (!C_Neg) {
	if (tempinC/1000)
		// 48 is the decimal character code value for displaying 0 on LCD
		tempC[0] = tempinC/1000 + 48;
	else tempC[0] = ' ';
	}
	tempC[1] = (tempinC/100)%10 + 48; // Extract tens digit
	tempC[2] = (tempinC/10)%10 + 48; // Extract ones digit

	// convert temp_fraction to characters
	tempC[4] = tempinC%10 + 48; // Extract tens digit

		PutString(tempC);

	if (!F_Neg) {
		if (tempinF/1000)
			tempF[0] = tempinF/1000 + 48;
		else tempF[0] = ' ';
	}

	tempF[1] = (tempinF/100)%10 + 48; // Extract tens digit
	tempF[2] = (tempinF/10)%10 + 48;
	tempF[4] = tempinF%10 + 48;
	PutString(tempF);
}*/

void DS18X2X_show_id_uart(unsigned char *id, char n )
{
	char i = 0;
	unsigned char fcmsg[] = "FC:";
	unsigned char crcmsg[] = "CRC:";
	unsigned char snmsg[] = "SN: ";
	unsigned char smsg[] = "(18S)";
	unsigned char bmsg[] = "(18B)";
	unsigned char xmsg[] = "(DS1822)";
	unsigned char qmsg[] = "( ? )";
	unsigned char crcfmsg[] = " CRC FAIL ";
	unsigned char crcokmsg[] = " CRC O.K. ";
	for( i = 0; i < n; i++ ) 
    { 
		if ( i == 0 ) PutString(fcmsg);
		else if ( i == n-1 ) PutString(crcmsg);
		if ( i == 1 ) PutString(snmsg);
		hex_ascii(id[i]);
		Putchar(0x20);
		if ( i == 0 ) 
       	{ 
			if ( id[0] == DS18S20_ID ) PutString(smsg);
         	else if ( id[0] == DS18B20_ID ) PutString(bmsg);
			else if(id[0] == DS1822_ID) PutString(xmsg);
         	else PutString(qmsg);
       	}
    }
   	if(crc8(id, OW_ROMCODE_SIZE))
      PutString(crcfmsg);
   	else 
      PutString(crcokmsg);
}

void ow_command(unsigned char command, unsigned char *id)
{ 
	unsigned char i = 0;
   	OW_reset_pulse();
	if(id)
	{
	   	OW_write_byte(MATCHROM_COMMAND_DS1820);			// to a single device
	    i = OW_ROMCODE_SIZE;
	    do 
	    {
			OW_write_byte(*id);
	        id++;
	    } 
	    while(--i);
	}
	else	// to all devices
	{
		OW_write_byte(IGNOREROM_COMMAND_DS1820);
	}
	OW_write_byte(command);
 }

unsigned char crc8 (unsigned char* data_in, unsigned int number_of_bytes_to_read)
{
	unsigned char crc = 0;
   	unsigned int loop_count = 0;
   	unsigned char  bit_counter = 0;
   	unsigned char  data = 0;
   	unsigned char  feedback_bit = 0;
	
	crc = CRC8INIT;
   	for (loop_count = 0; loop_count != number_of_bytes_to_read; loop_count++)
   	{
		data = data_in[loop_count];
      	bit_counter = 8;
      	do 
       	{ 
			feedback_bit = (crc ^ data) & 0x01;
         	if(feedback_bit == 0x01 ) 
            	crc = crc ^ CRC8POLY;
         	crc = (crc >> 1) & 0x7F;
         	if ( feedback_bit == 0x01 ) 
            	crc = crc | 0x80;
         	data = data >> 1;
         	bit_counter--;
       	} 
      	while (bit_counter > 0);
	}
	return crc;
}
/**********************************************************************
* Function:        unsigned char Detect_Slave_Device(void)
* PreCondition:    None
* Input:		   None	
* Output:		   1 - Not Present   0 - Present 	
* Overview:   	   To check the presence of slave device.    
***********************************************************************/

unsigned char Detect_Slave_Device(void)
{
	if (!OW_reset_pulse())
		return HIGH;
	else 		
		return LOW;
}	



/**********************************************************************
* Function:        void wait (unsigned char delay_time)
* PreCondition:    None
* Input:		   Required Delay value	
* Output:
* Side Effects:
* Overview:
*					Function delay_5Us with clock 25MHz;
*					calculation: 
*					20*0x16 + (7+n)*0.16*delay_time in Us
*					20	: single instructions including call and return.
*					n   : 1 (number of NOPs).
*					8	: (7 + n) ....7 Instructions used by 'for' loop and 1 nop instruction.
* 					0.16: (1/25000000)*4 ...Clock 25MHz
* 					delay_needed in Us.
* 					Calculated delay_time_value in Us.
*					delay_time_value = (delay_needed - (20*0.16))/(8*0.16)
*					delay_time_value = (delay_needed - 3.2) / (1.28)
************************************************************************/
//#ifdef FOR_LOOP_DELAY
//void wait (unsigned char delay_time)
//{
//	unsigned char temp;
//	
//	for(temp = 0; (temp<(delay_time)); temp++)
//	_asm nop _endasm
//}	
//#endif

/**********************************************************************
* Function:        void hex_ascii(unsigned char display_data)
* PreCondition:    None
* Input:		   HEX Value 
* Output:		   None	 
* Side Effects:
* Overview:		 Convert Hex Value to ASCII and send to PC;
***********************************************************************/
void hex_ascii(unsigned char display_data)
{
	unsigned char temp;
	temp = ((display_data & 0xF0)>>4);
	if (temp <= 0x09)
		Putchar(temp+'0');	
	else
		Putchar(temp+'0'+0x07);	

	temp = display_data & 0x0F;
	if (temp <= 0x09)
		Putchar(temp+'0');	
	else
		Putchar(temp+'0'+0x07);	
	
//	Putchar('\r');
//	Putchar('\n');
}	
	
/**********************************************************************
* Function:        void initEUSART (void)
* PreCondition:    None
* Input:		   None	
* Output:		   None
* Side Effects:
* Overview:		  Initialization of USART 
					1. Enable Transmitter and select High Baud Rate in TXSTA
					2. Select a value in SPGRB register to generate 9600 baud rate 
					3. Enable Serial port in RCSTA register
					4. Enable continuous reception in RCSTA

***********************************************************************/
void initEUSART (void)
{
	  //Configure RC6/TX/CK and RC7/RX/DT as a USART:
	  UART_RX_DIRECTION = INPUT;
	  UART_TX_DIRECTION = OUTPUT;
	  
 	  UART_TX_STATUS = CLEAR;           		// Reset TXSTA 
	  UART_RX_STATUS = CLEAR;		   			// Reset RCSTA	

 	  UART_TX_STATUS = 0x20;		    		// Load TXSTA from TxConUser selected)
	  SPBRG = BAUD_RATE;       					// Write baudrate to SPBRG
	  UART_RX_STATUS = 0x90;		    
	  BAUDCON = BAUD_CONTROL;
   /**************************************************************************************************/
}

/**********************************************************************
* Function:        void PutString(unsigned char *mess)
* PreCondition:    None
* Input:		   Pointer to display message	
* Output:		   None
* Side Effects:
* Overview:		   Sends the string to terminal till it finds NULL character.
***********************************************************************/
void PutString(unsigned char *mess)
{
	while(*mess != '\0')
	{
  		Putchar(*mess++);
	}
}

/**********************************************************************
* Function:        unsigned char Getchar(void)
* PreCondition:    None
* Input:		   None	
* Output:		   Return a byte received from UART	
* Side Effects:
* Overview:		  Wait till a byte received from termianl and then returns the received data	
***********************************************************************/
unsigned char Getchar(void)
{
	unsigned char recv_data;

	while(!PIR1bits.RCIF);
	recv_data = RCREG;
	PIR1bits.RCIF = CLEAR;
}

/**********************************************************************
* Function:        void Putchar (unsigned char send_data)
* PreCondition:    None
* Input:		   Byte to transmit		
* Output:		   None		
* Side Effects:
* Overview:		   Transmits a byte to terminal through UART.	
***********************************************************************/
void Putchar (unsigned char send_data)
{ 
  	TXREG = send_data;	// Also transmit USART status value
	while (!TXSTAbits.TRMT);	// wait here transmission to complete.	
}

/********************************************************************************************
                  E N D     O F     M A I N   
*********************************************************************************************/